//--------------------------------------------------
// Projet DUINO303
// Module gestion MIDI -> CV/GATE 
// pour alimenter un module x0x heart
//--------------------------------------------------
// Frederic ABIGNOLI 
// v0.8 : 22/03/2014
// v1.0 : 30/01/2021
//        Modification code pour IDE recent
//		  Utilisation librairie MIDI 5.0.2
//        Redefinition des PROGMEM
//		  Registres PORT a la place des digitalwrite
//--------------------------------------------------


// --------------------------------
// declaration des pins sur ARDUINO
// --------------------------------
#define MIDIIN  	0   // Entree MIDI
#define MIDIOUT 	1   // Sortie MIDI
#define SQUAREON 	2	// Gestion square on/off
#define SAWON		3	// Gestion saw on/off
#define SLIDEINPUT	4	// Gestion slide input on/off
#define SLIDEOUTPUT	5	// Gestion slide output on/off
#define CUTOFF		6	// Pin A1 en numerique gestion cutoff
#define DECAYON		7	// Gestion decay on/off
#define GATEOUT		8   // Gestion gate out
// PINS D9 a D13 reservees SPI
//------------------------------------------------------
#define DAC1		9   // Pin D9 pour DAC no 1
#define SPI-SS		10	// Chip select
#define SPI-MOSI	11	// MOSI
#define SPI-MISO	12	// MISO
#define SPI-SCK		13	// SCK

// Pin A0 a A5
//------------
#define BNAV		0   // Pin A0 : Boutons Fonctions, entree analogique 0
#define ACCENTON	15	// Pin A1 : Gestion accent on/off
#define ENVMOD		16	// Pin A2 : Gestion envmod
#define NOUSED1		17	// Pin A3 non utilisee
#define NOUSED1		18	// Pin A4 non utilisee
#define NOUSED1		19	// Pin A5 non utilisee

// Control change
//---------------
#define CTRLOSC 	110	// Wave saw, square, saw+square
#define CTRLSLIDE 	111	// Slide input, output, off
#define CTRLCUTOFF 	112	// Cutoff on-off
#define CTRLDECAY 	113	// Decay on-off
#define CTRLACCENT 	114	// Accent on-off
#define CTRLENVMOD	115	// Envmod on-off

//---------------------
// Gestion des switches
//---------------------
#define B_WAVE		1   // Touche selection oscillateur
#define B_SLIDE		2   // Touche selection slide
#define B_CUTOFF	3   // Touche activation cutoff
#define B_DECAY		4   // Touche activation decay
#define B_ACCENT	5   // Touche activation accent
#define B_ENVMOD	6   // Touche actication envmod
#define B_MAINT		9   // Derniere touche maintenue appuyee
#define DEBOUNCE	20	// Debounce en ms pour les boutons sur entree analogique
byte lastbouton; 		// Stockage du numero de dernier bouton appuye
int valbouton;			// Valeur du bouton appuye

// Function states
//----------------
byte v_wave;			// Oscillateur (0: rien, 1:square, 2:saw, 3:square+saw
byte v_slide;			// Slide (0: rien, 1: juste en input, 2: juste en output)
byte v_decay;			// Decay (0: inactif, 1: actif)
byte v_accent;			// Accent (0: inactif, 1: actif)
byte v_cutoff;			// Cutoff (0: inactif, 1: actif)
byte v_envmod;			// Envmod (0: inactif, 1: actif)

// Divers
//-------
#define ON  1
#define OFF 0

//------------------------------------------------
// Bibliotheque et declarations pour gerer le midi
//------------------------------------------------
#include <MIDI.h>   // Midi Library
MIDI_CREATE_DEFAULT_INSTANCE();
#define MONO    0   // note on sur note active -> pas de GATE 
#define RETRIG  1   // note on sur note actice -> redeclenchement dun GATE
byte CountNote;     // Nombre de note appuyees sur le clavier a un instant "T"
byte ModeMidi;      // Mode MONO ou RETRIG
#define BLANCHES  0 // Notes blanches
#define SEMITONES 1 // Toutes les notes

//------------------------------------------------
// declarations pour gerer les DAC
//------------------------------------------------
int vdac;				// valeur a ecrire dans les DAC
int lastvdac=0;			// Stockage ancienne valeur vdac
int MidiChannel=1;
//----------------------------------
// Tableaux de conversion midi -> CV
//----------------------------------
#include "valeursmem.h" 	// Valeur DAC suivant note midi


//---------------------
// Declaration diverses
//---------------------
int i,j;              // Variables pour les boucles
//--------------------
// SETUP DU PROGRAMME
//--------------------
void setup()
{
	MidiChannel=1;

	// Initialisation des pins
	//------------------------ 
	pinMode(SQUAREON,OUTPUT);		// Pin commande oscillateur square
	pinMode(SAWON,OUTPUT);			// Pin commande oscillateur saw
	pinMode(SLIDEINPUT,OUTPUT);		// Pin commande du slide en input
	pinMode(SLIDEOUTPUT,OUTPUT);	// Pin commande du slide en output
	pinMode(DECAYON,OUTPUT);		// Pin commande du decay
	pinMode(ACCENTON,OUTPUT);		// Pin commande de l'accent
	pinMode(GATEOUT,OUTPUT);		// Pin commande du gate
	pinMode(CUTOFF,OUTPUT);			// Pin commande du cutoff
	pinMode(ENVMOD,OUTPUT);			// Pin commande du envmod
	pinMode(10, OUTPUT);			// SS
	pinMode(DAC1,OUTPUT);			// Commande du DAC
	
	ChooseMidiChannel();
	// Initialisation du MIDI
	//-----------------------
	MIDI.begin(MidiChannel);              // demarre le midi en mode OMNI
	CountNote=0;                                // Compteur de note actives
	
	// Initialisation Handler Midi
	//----------------------------
	MIDI.setHandleNoteOn(MyHandleNoteOnCV);       	// handler note on 
	MIDI.setHandleNoteOff(MyHandleNoteOffCV);     	// handler note off
	MIDI.setHandleControlChange(MyHandleCtrlCV);  	// handler CC
	PORTB &= ~(1<<0);								// Etat de base gate off
	ModeMidi=RETRIG;								// Retrig du gate a chaque note on
	
	// Preparation SPI
	// et initialisation du DAC
	SPCR = 0x50;			// Enable SPI in Master mode
    SPSR = 0x01;			// enable SPI2x
    DDRB |= 0x2E;			// Set pin 9,10,11,13 as output
    PORTB |= (1<<1);		// Chip Select	(pin 9 = PORTB bit 1)
	writeDAC(0);			// Ecrire 0 dans le dac (sa sortie est a 0V)
	
	// Initialisation des commandes de la x0x
	//---------------------------------------

	// Mode SAW 
	v_wave=2;
	refresh_wave();
	// mode no slide
	v_slide=0;
	refresh_slide();
	// Mode decay off
	v_decay=0;
	refresh_decay();
	// Mode accent on
	v_accent=1;
	refresh_accent();
	// Mode cutoff off
	v_cutoff=0;
	refresh_cutoff();
	// Mode envmod off
	v_envmod=0;
	refresh_envmod();
	
}

//--------------------
// PROGRAMME PRINCIPAL
//--------------------
void loop() {
  MIDI.read();
  
  gestionBoutons();
}
