//------------------------------
// Procedures pour gerer le midi
//------------------------------

//-----------------------------------------
// Handler CC MIDI
// Gere la selection de forme d'onde
// Le slide, le decay, l'accent
// le cutoff et l'envmod
//----------------------------------------
void MyHandleCtrlCV(byte channel, byte number, byte value) {
	switch(number) {
		// Control change for wave select : saw, square, saw+square
		case CTRLOSC :
			if (value <40)					// Square
				v_wave=1;
			else
				if (value>40 && value <80)	// Saw
					v_wave=2;
				else
					v_wave=3;				// Saw + square
			refresh_wave();
			break;
		// Control change for slide input, output or not
		case CTRLSLIDE :
			if (value <40)					// No slide
				v_slide=0;
			else
				if (value>40 && value <80)	// Slide output
					v_slide=2;
				else
					v_slide=1;				// Slide input
			refresh_slide();
			break;
		
		// Control change for filter cutoff
		case CTRLCUTOFF :
			if (value <64)					// Cutoff off
				v_cutoff=0;
			else
				v_cutoff=1;					// Cutoff on
			refresh_cutoff();
			break;
		// Control change for decay
		case CTRLDECAY :
			if (value <64)					// Decay off
				v_decay=0;
			else
				v_decay=1;					// Decay on
			refresh_decay();
			break;
			
		// Control change for accent
		case CTRLACCENT :
			if (value <64)					// Accent off
				v_accent=0;
			else
				v_accent=1;					// Accent on
			refresh_accent();
			break;
		// Control change for envmod
		case CTRLENVMOD:
			if (value <64)					// Envmod off
				v_envmod=0;
			else
				v_envmod=1;					// Envmod on
			refresh_envmod();
			break;
			
	}
	
}


//----------------------------- 
// HANDLER NOTE OFF MIDI -> CV
//----------------------------- 
void  MyHandleNoteOffCV(byte channel, byte pitch, byte velocity) {
	if (CountNote) 			// Si countnote >0
		CountNote--;		// decrementer jusqu'a 0 si besoin
    if (!CountNote)         // S'il ne reste plus de note, mettre GATE a zero
 		PORTB &= ~(1<<0);	// GATE ON
}
                                                                                             
//---------------------------- 
// HANDLER NOTE ON MIDI -> CV
//---------------------------- 
void MyHandleNoteOnCV(byte channel, byte pitch, byte velocity) {
	// Si velocity == 0 alors on a recu un note off
	//---------------------------------------------
	if (velocity==0) {
		MyHandleNoteOffCV(channel,pitch,velocity);
		return;
	}
	vdac=0;
	if (pitch>=36 and pitch <=96) {
		vdac = pgm_read_word_near( VOct + pitch - 36);
	} 
	else
		return;

	CountNote++;    // Incrementation du nombre de note actives

	// Si mode RETRIG et qu'une note etait deja appuyee avant celle en cours
	// il faut redeclencher le gate donc on le passe à 0
	if (ModeMidi==RETRIG and CountNote>1) {
		PORTB &= ~(1<<0);	// GATE OFF
		delay(5);
	}
	// Ecriture dans le DAC1 de la hauteur de note si elle an change
	// en cas de repetition, on ne la reecrit pas dans le dac
	if (lastvdac!=vdac) {
		writeDAC(vdac);
		lastvdac=vdac;
	}
  // On ouvre le GATE pour jouer la note en cours
  if (ModeMidi==MONO and CountNote==1)
		PORTB |= (1<<0);	// GATE ON

  if (ModeMidi==RETRIG)
		PORTB |= (1<<0);	// GATE ON
}

void TestMidiChannel(void) {
	valbouton=readButtons(BNAV);
    if (valbouton==0 || valbouton==B_MAINT)     // Si aucun bouton appuye on sort
        return;           // pour ne pas perdre de temps
	MidiChannel=valbouton;
}

void ChooseMidiChannel(void) {
int i,j;
	for (i=0; i<4; i++) {
		TestMidiChannel();
		switch(MidiChannel) {
			case 1:
				for (j=0; j<3; j++) {
					digitalWrite(SQUAREON, HIGH);
					delay(100);
					digitalWrite(SQUAREON, LOW);
					delay(100);
				}
				break;
			case 2:
				for (j=0; j<3; j++) {
					digitalWrite(SLIDEOUTPUT, HIGH);
					delay(100);
					digitalWrite(SLIDEOUTPUT, LOW);
					delay(100);
				}
				break;
			case 3:
				for (j=0; j<3; j++) {
					digitalWrite(CUTOFF, HIGH);
					delay(100);
					digitalWrite(CUTOFF, LOW);
					delay(100);
				}
				break;
			case 4:
				for (j=0; j<3; j++) {
					digitalWrite(DECAYON, HIGH);
					delay(100);
					digitalWrite(DECAYON, LOW);
					delay(100);
				}
				break;
			case 5:
				for (j=0; j<3; j++) {
					digitalWrite(ACCENTON, HIGH);
					delay(100);
					digitalWrite(ACCENTON, LOW);
					delay(100);
				}
				break;
			case 6:
				for (j=0; j<3; j++) {
					digitalWrite(ENVMOD, HIGH);
					delay(100);
					digitalWrite(ENVMOD, LOW);
					delay(100);
				}
				break;
		}
	}
}



